<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco;

use Financeiro\Entities\Boletreceber;
use Geral\Entities\Banco;
use Geral\Entities\Endeclie;
use Geral\Entities\Contcorr;
use Financeiro\Util\Util;
use Geral\Entities\Empresa;
use Teknisa\Libs\Util\DateUtil;
use Geral\Entities\Licencauso;
use Geral\Entities\Cliente;
use Geral\Entities\Filial;
use Geral\Entities\Municipio;

use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;

class GeraCnabSicrediService extends GeraCnabService{

    const IDENTIFICACAO_REGISTRO = '0';
    const IDENTIFICACAO_FITA_REMESSA = '1';
    const IDENTIFICACAO_MOVIMENTO = 'REMESSA';
    const IDENTIFICACAO_TIPO_SERVICO = '01';
    const IDENTIFICACAO_POR_EXTENSO_TIPO_SERVICO = 'COBRANCA';
    const IDENTIFICACAO_REGISTRO_TRANSACAO = '1';
    const TIPO_INSCRICAO_EMPRESA_CPF = '01';
    const TIPO_INSCRICAO_EMPRESA_CGC = '02';
    const CODIGO_BANCO = '748';
    const CODIGO_CARTEIRA = 'I';
    const IDENTIFICACAO_REGISTRO_TRAILLER = '9';
    const VERSAO_ARQUIVO = '2.00';
    const NOME_BANCO = 'SICREDI';

    public function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL, $contreg = NULL, $idtipcnabrec = NULL, $cdfilial, $contador = NULL){
        try {
            if (!is_dir($caminho))
                mkdir($caminho, "0777", true);
            $nmarquivo = $caminho.$nmarquivo;
            $file = fopen($nmarquivo, "w+");

            // CAMPO: Identificação do Registro
            // POSIÇÃO: 001 a 001
            // PICTURE: 9(1)
            $arquivo = self::IDENTIFICACAO_REGISTRO;

            // CAMPO: Identificação da fita remessa
            // POSIÇÃO: 002 a 002
            // PICTURE: 9(1)
            $arquivo .= self::IDENTIFICACAO_FITA_REMESSA;

            // CAMPO: Identificação por extenso do movimento
            // POSIÇÃO: 003 a 009
            // PICTURE: X(7)
            $arquivo .= self::IDENTIFICACAO_MOVIMENTO;

            // CAMPO: Identificação do tipo de serviço
            // POSIÇÃO: 010 a 011
            // PICTURE: 9(2)
            $arquivo .= self::IDENTIFICACAO_TIPO_SERVICO;

            // CAMPO: Identificação por extenso do tipo de serviço
            // POSIÇÃO: 012 a 026
            // PICTURE: X(15)
            $arquivo .= str_pad(self::IDENTIFICACAO_POR_EXTENSO_TIPO_SERVICO, 15, ' ');
            
            // CAMPO: Código do Cedente
            // POSIÇÃO: 27 a 31
            // PICTURE: 9(5)
            // CONTEUDO: CCCCC
            $contcorr = Contcorr::findOneByCdreducont($cdreducont);
            if($contcorr->getCdcedenteban()){
                $arquivo .= str_pad(substr($contcorr->getCdcedenteban(), -5), 5, '0', STR_PAD_LEFT);
            }else{
                $arquivo .= str_pad($arquivo, 31, '0');
            }
    
            // CAMPO: Nº de inscrição da empresa/sacador
            // POSIÇÃO: 32 a 45
            // PICTURE: 9(14)
            $empresa = Empresa::findOneByCdempresa($cdempresa);
            if($empresa)
                $arquivo .= str_pad($empresa->getNrinsjurempr(), 14, '0', STR_PAD_LEFT);
            
            // CAMPO: Brancos
            // POSIÇÃO: 46 a 76
            // PICTURE: X(273)
            $arquivo = str_pad($arquivo, 76);

            // CAMPO: Código do Banco
            // POSIÇÃO: 077 a 079
            // PICTURE: 9(3)
            $arquivo .= self::CODIGO_BANCO;

            // CAMPO: Nome do Banco
            // POSIÇÃO: 080 a 094
            // PICTURE: X(15)
            $arquivo .=  str_pad(self::NOME_BANCO, 15);

            // CAMPO: Data de Gravação da Fita
            // POSIÇÃO: 095 a 102
            // PICTURE: 9(8)
            $now = DateUtil::getDataAtual();
            $data = DateUtil::getAno($now) .  DateUtil::getMes($now) . DateUtil::getDia($now);
            $arquivo .= Util::campoCnabValido($data);
            
            // CAMPO: Brancos
            // POSIÇÃO: 103 a 110
            // PICTURE: X(273)
            $arquivo = str_pad($arquivo, 110);
            
            // CAMPO: Número Seqüencial
            // POSIÇÃO: 111 a 117
            // PICTURE: 9(7)
            $arquivo .= str_pad($contreg, 7, "0", STR_PAD_LEFT);
            
            // CAMPO: Brancos
            // POSIÇÃO: 118 a 390
            // PICTURE: X(273)
            $arquivo = str_pad($arquivo, 390);
            
            // CAMPO: Versão do Arquivo
            // POSIÇÃO: 391 a 394
            // PICTURE: X(4)
            $arquivo .= self::VERSAO_ARQUIVO;
            
            // CAMPO: Nº seq. do reg. no lote
            // POSIÇÃO: 395 a 400
            // PICTURE: 9(5)
            // Número de seqüencia do registro no lote inicializado sempre em 1
            $contador = str_pad($contador, 6, "0", STR_PAD_LEFT);
            $arquivo .= Util::campoCnabValido($contador) . "\r\n";

            //Grava a variavel arquivo no arquivo texto
            fprintf($file, "%s", $arquivo);
            fclose($file);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
                                  $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
                                  $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
                                  $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL)
    {
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do registro detalhe
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        // CONTEÚDO: fixo '1'
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRANSACAO;

        // CAMPO: Tipo de Cobrança
        // POSIÇÃO: 002 a 002
        // PICTURE: X(1)
        // CONTEÚDO: fixo 'A' - SICREDI com Registro
        $arquivo .= 'A';

        // CAMPO: Tipo de carteira
        // POSIÇÃO: 003 a 003
        // PICTURE: X(1)
        // CONTEÚDO: fixo 'A' - Simples
        $arquivo .= 'A';
        
        // CAMPO: Tipo de Impressão
        // POSIÇÃO: 004 a 004
        // PICTURE: X(1)
        // CONTEÚDO: A - NORMAL    B - CARNÊ
        $arquivo .= 'A';

        // CAMPO: Filler
        // POSIÇÃO: 005 a 016
        // PICTURE: X(12)
        // CONTEÚDO: Brancos
        $arquivo = str_pad($arquivo, 16);

        // CAMPO: Tipo da Moeda
        // POSIÇÃO: 017 a 017
        // PICTURE: X(1)
        // CONTEÚDO: Fixo 'A' - REAL
        $arquivo .= 'A';
        
        // CAMPO: Tipo de Desconto
        // POSIÇÃO: 018 a 018
        // PICTURE: X(1)
        // CONTEUDO: A - VALOR    B - PERCENTUAL
        $arquivo .= 'A';
        
        // CAMPO: Tipo de Juros //Não é mora...
        // POSIÇÃO: 019 a 019
        // PICTURE: X(1)
        // CONTEÚDO: A - VALOR    B - PERCENTUAL
        $arquivo .= 'A';
        
        // CAMPO: Filler
        // POSIÇÃO: 020 a 047
        // PICTURE: X(28)
        // CONTEÚDO: Brancos
        $arquivo = str_pad($arquivo, 47);
        
        // CAMPO: Nosso Número
        // POSIÇÃO: 048 a 056
        // PICTURE: 9(9)
        //stLinha := stLinha + Ajusta(Copy(NossoNumero(Nr_Codigo(cdsBoletosCDNOSNUMREC.AsString)), 1, 9), 9);
        if($cdnosnumrec)
             $arquivo .= str_replace([' ', '-'], '', Util::campoCnabValido($cdnosnumrec));
        else $arquivo = str_pad($arquivo, 56, '0');
        
        // CAMPO: Filler
        // POSIÇÃO: 057 a 062
        // PICTURE: X(6)
        // CONTEÚDO: Brancos
        $arquivo = str_pad($arquivo, 62);
        
        // CAMPO: Data da Instrução
        // POSIÇÃO: 063 a 070
        // PICTURE: 9(8)
        // CONTEÚDO: AAAAMMDD
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getAno($now) .  DateUtil::getMes($now) . DateUtil::getDia($now);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: Complemento para instrução (OPCIONAL)
        // POSIÇÃO: 071 a 071
        // PICTURE: X(8)
        // CONTEÚDO: Fixo Brancos
        $arquivo .= ' ';
        
        // CAMPO: Postagem do Título
        // POSIÇÃO: 072 a 072
        // PICTURE: X(1)
        // CONTEÚDO: S - POSTAR TÍTULO    N -  NÃO POSTAR E REMETER PARA O CEDENTE
        if(($idimprboleto == 'B') || (!$cdnosnumrec)){
            $arquivo .= 'S';
        }else{
            $arquivo .= 'N';
        }
        
        // CAMPO: Filler
        // POSIÇÃO: 073 a 073
        // PICTURE: X(1)
        $arquivo .= ' ';
        
        // CAMPO: Emissão de bloqueto
        // POSIÇÃO: 074 a 074
        // PICTURE: X(1)
        if(($idimprboleto == 'B') || (!$cdnosnumrec)){
            $arquivo .= 'A';// BANCO EMITE
        }else{
            $arquivo .= 'B';// CLIENTE EMITE
        }
        
        // CAMPO: Número da Parcela no carnê (OPCIONAL)
        // POSIÇÃO: 075 a 076
        // PICTURE: 9(2)
        $arquivo .= '00';
        
        // CAMPO: Número total de parcelas no carnê (OPCIONAL)
        // POSIÇÃO: 077 a 078
        // PICTURE: 9(2)
        $arquivo .= '00';
        
        // CAMPO: Filler
        // POSIÇÃO: 079 a 082
        // PICTURE: X(4)
        $arquivo = str_pad($arquivo, 82);
        
        // CAMPO: Valor de desconto por dia de antecipação
        // POSIÇÃO: 083 a 092
        // PICTURE: 9(10)
        $arquivo = str_pad($arquivo, 92, '0');
        
        // CAMPO: % multa por pagamento em atraso
        // POSIÇÃO: 093 a 096
        // PICTURE: 9(4)
        if(!$vrmordiabrec){
            if(!$vrpemobrec){
                $arquivo = str_pad($arquivo, 96, '0');
            }
            else{
                $arquivo .= str_pad(str_replace('.', '', ($vrfinalbrec * $vrpemobrec) / 100), 13, '0', STR_PAD_LEFT);
            }
        }
        else{
            $arquivo .= str_pad(str_replace('.', '', $vrmordiabrec * 100), 13, '0', STR_PAD_LEFT);
        }
        
        // CAMPO: Filler
        // POSIÇÃO: 097 a 108
        // PICTURE: 9(12)
        // Conteúdo: Brancos
        $arquivo = str_pad($arquivo, 108);
        
        // CAMPO: Instrução
        // POSIÇÃO: 109 a 110
        // PICTURE: 9(2)
        if($cdins1cobbol)
            $arquivo .= $cdins1cobbol;
        else
            $arquivo = str_pad($arquivo, 110, '0');
        
        // CAMPO: Seu Número
        // POSIÇÃO: 111 a 120
        // PICTURE: X(10)
        $arquivo .= str_pad($nrlanbolrec, 10);
        
        // CAMPO: Data de Vencimento
        // POSIÇÃO: 121 a 126
        // PICTURE: 9(6)
        // CONTEUDO: DDMMAA
        $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $data = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . substr(DateUtil::getAno($vencimento), -2);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: Valor Principal do título
        // POSIÇÃO: 127 a 139
        // PICTURE: 9(13)
        $vr = $vrfinalbrec * 100;
        $vr = str_pad($vr, 13, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vr);
        
        // CAMPO: Filler
        // POSIÇÃO: 140 a 148
        // PICTURE: X(9)
        $arquivo = str_pad($arquivo, 148);
        
        // CAMPO: Espécie de Documento
        // POSIÇÃO: 149 a 149
        // PICTURE: X(1)
        // CONTEUDO: A - Duplicata Mercantil por indicação     B - Duplicata Rural    C - Nota Promissória                      D - Nota Promissoria Rural
        //           E - Nota de Seguros                       G - Recibo             H - Letra Cambio
        //           I - Nota de Débito                        J - Duplicata de Serviço por Indicação
        //           K - Outros
        $arquivo .= 'A';
        
        // CAMPO: Aceite do Título
        // POSIÇÃO: 150 a 150
        // PICTURE: X(1)
        // CONTEÚDO: S - SIM    N - NÃO
        $arquivo .= 'S';
        
        // CAMPO: Data de Emissão
        // POSIÇÃO: 151 a 156
        // PICTURE: 9(6)
        // CONTEUDO: DDMMAA
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($vencimento), -2);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: Instrução de protesto automático
        // POSIÇÃO: 157 a 158
        // PICTURE: 9(2)
        if(!$cddiaprotbol)
            $arquivo .= '00';
        else
            $arquivo .= '06';
        
        // CAMPO: Número de dias para protesto
        // POSIÇÃO: 159 a 160
        // PICTURE: 9(2)
        if($qtdiaprotbol)
            $arquivo .= $qtdiaprotbol;
        else
            $arquivo .= '00';
        
        // CAMPO: Valor / % de juros por dia de atraso
        // POSIÇÃO: 161 a 173
        // PICTURE: 9(13)
        if(!$vrmordiabrec){
            if(!$vrpemobrec){
                $arquivo = str_pad($arquivo, 173, '0');
            }else{
                $arquivo .= str_pad(str_replace('.', '', ($vrfinalbrec * $vrpemobrec) / 100), 13, '0', STR_PAD_LEFT);
            }
        }else{
            $arquivo .= str_pad(str_replace('.', '', $vrmordiabrec * 100), 13, '0', STR_PAD_LEFT);
        }
        
        // CAMPO: Data limite para concessão de desconto (Opcional)
        // POSIÇÃO: 174 a 179
        // PICTURE: 9(6)
        // CONTEUDO: DDMMAA
        $arquivo = str_pad($arquivo, 179, '0');
        
        // CAMPO: Valor / % do desconto
        // POSIÇÃO: 180 a 192
        // PICTURE: 9(13)
        $arquivo = str_pad($arquivo, 192, '0');
        
        // CAMPO: Filler
        // POSIÇÃO: 193 a 205
        // PICTURE: 9(13)
        // CONTEUDO: Zeros
        $arquivo = str_pad($arquivo, 205, '0');
        
        // CAMPO: Valor do abatimento
        // POSIÇÃO: 206 a 218
        // PICTURE: 9(13)
        $arquivo = str_pad($arquivo, 218, '0');
        
        // ###############################################
        // VERIFICA LICENÇA DE USO DO CLIENTE  ###########
        // ###############################################
        $nrInsJur = 0;
        $nrCepClie = 0;
        $nrInsJur = str_pad($nrInsJur, 14, '0');
        $nrCepClie = str_pad($nrCepClie, 8, '0');
        /** @var $entitieLincenca \Geral\Repositories\Licencauso */
        //$entitieLincenca = Licencauso::getRepository();
        $licenca = null;//$entitieLincenca->getLicencaUso($cdfilial, $nrlanbolrec);
        if ($licenca){
            if ($licenca[0]['IDTPIJURLUSO'] == 'F')
                $tpInscSacado = '01'; //CPF
            else
                $tpInscSacado = '02'; //CPF

            if ($licenca[0]['NRINSJURLUSO'])
                $nrInsJur = $licenca[0]['NRINSJURLUSO'];

            if ($licenca[0]['NRCEPLUSO']){
                $nrCepClie = $licenca[0]['NRCEPLUSO'];
                $nmClie = $licenca[0]['NMFANTLUSO'];
                $dsEndeclie = $licenca[0]['DSENDELUSO'];
                $nmBairro = $licenca[0]['NMBAIRLUSO'];
                $nmMunicipio = $licenca[0]['NMMUNICIPIO'];
                $sgEstado = $licenca[0]['SGESTADO'];
            }
        }else {
            /** @var $entitieCliente Cliente */
            $entitieCliente = Cliente::findOneByCdcliente($cdcliente);
            if($entitieCliente){
                if($entitieCliente->getIdtpijurclie() == 'F')
                    $tpInscSacado = '1';
                else
                    $tpInscSacado = '2';

                if ($entitieCliente->getNrinsjurclie())
                    $nrInsJur = $entitieCliente->getNrinsjurclie();
                /** @var $entitieEndeClie Endeclie */
                $entitieEndeClie = Endeclie::findOneByCdclienteAndIdtpendeclie($cdcliente, 'C');
                
                if($entitieEndeClie){
                    if($entitieEndeClie->getNrcepclie())
                        $nrCepClie = $entitieEndeClie->getNrcepclie();
                    $nmClie = $entitieCliente->getNmrazsocclie();
                    $dsEndeclie = $entitieEndeClie->getDsendeclie();
                    $nmBairro = $entitieEndeClie->getNmbairclie();
                    $municipio = Municipio::findOneByNrorgAndCdmunicipio($nrorg, $entitieEndeClie->getCdMunicipio());
                    $nmMunicipio = $municipio->getNmmunicipio();
                    $sgEstado = $entitieEndeClie->getSgestado();
                }
            }
        }
        
        // CAMPO: Tipo de pessoa do sacado
        // POSIÇÃO: 219 a 219
        // PICTURE: 9(1)
        // CONTEUDO: 1 - CPF    2 - CNPJ
        $arquivo .= $tpInscSacado;
        
        // CAMPO: Filler
        // POSIÇÃO: 220 a 220
        // PICTURE: X(1)
        // CONTEUDO: Zeros
        $arquivo .= '0';
        
        // CAMPO: CIC/CGC do Sacado
        // POSIÇÃO: 221 a 234
        // PICTURE: 9(14)
        $nrInsJur = substr(Util::campoCnabValido($nrInsJur), 0, 14);
        $arquivo .= str_pad($nrInsJur, 14);
        
        // CAMPO: Nome do Sacado
        // POSIÇÃO: 235 a 274
        // PICTURE: X(40)
        $nmClie = substr(Util::campoCnabValido($nmClie), 0, 40);
        $arquivo .= str_pad($nmClie, 40);
        
        // CAMPO: Endereço do Sacado
        // POSIÇÃO: 275 a 314
        // PICTURE: X(40)
        $dsEndeclie = substr(Util::campoCnabValido($dsEndeclie), 0, 40);
        $arquivo .= str_pad($dsEndeclie, 40);
        
        // CAMPO: Código do Sacado na Cooperativa
        // POSIÇÃO: 315 a 319
        // PICTURE: 9(5)
        $arquivo = str_pad($arquivo, 319, '0');
        
        // CAMPO: Filler
        // POSIÇÃO: 320 a 325
        // PICTURE: 9(6)
        // CONTEUDO: Zeros
        $arquivo = str_pad($arquivo, 325, '0');
        
        // CAMPO: FILLER
        // POSIÇÃO: 326 a 326
        // PICTURE: X(1)
        // CONTEUDO: BRANCOS
        $arquivo .= ' ';
        
        // CAMPO: CEP do Sacado
        // POSIÇÃO: 327 a 334
        // PICTURE: 9(8)
        $arquivo .= Util::campoCepTel($nrCepClie);
        
        // CAMPO: Código do Sacado junto ao cliente
        // POSIÇÃO: 335 a 339
        // PICTURE: 9(5)
        $arquivo = str_pad($arquivo, 339, '0');
        
        // CAMPO: CIC/CGC do Avalista (Opcional)
        // POSIÇÃO: 340 a 353
        // PICTURE: 9(14)
        // CONTEÚDO: Zeros
        $arquivo = str_pad($arquivo, 353, '0');
        
        // CAMPO: Nome do Sacador avalista (Opcional)
        // POSIÇÃO: 354 a 394
        // PICTURE: X(41)
        // CONTEÚDO: Brancos
        $arquivo = str_pad($arquivo, 394);
        
        // CAMPO: Número sequencial do registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $contador = str_pad($contador, 6, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    
    public function geraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do registro
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRAILLER;
        
        // CAMPO: Identificação do arquivo remessa
        // POSIÇÃO: 002 a 002
        // PICTURE: 9(3)
        $arquivo .= self::IDENTIFICACAO_FITA_REMESSA;
        
        // CAMPO: Número do Sicredi
        // POSIÇÃO: 003 a 005
        // PICTURE: 9(3)
        $arquivo .= self::CODIGO_BANCO;
        
        // CAMPO: Código do Cedente
        // POSIÇÃO: 006 a 010
        // PICTURE: 9(5)
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr->getCdcedenteban()){
            $arquivo .= str_pad(substr($contcorr->getCdcedenteban(), -5), 5, '0', STR_PAD_LEFT);
        }else{
            $arquivo .= str_pad($arquivo, 31, '0');
        }

        // CAMPO: Brancos
        // POSIÇÃO: 002 a 394
        // PICTURE: X(393)
        $arquivo = str_pad($arquivo, 394);

        // CAMPO: Número Seqüencial
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT);

        $arquivo.=  "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    public function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial){}
    public function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
                                         $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero,
                                         $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec,
                                         $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol,
                                         $qtdiaprotbol, $contador = NULL){}
    public function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL){}
    public function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL){}
    public function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL){}


}